@@ -314,7 +314,7 @@ class OptionParser(object):
             i += 1
 
             if not option:
-                self._error('no such option: %s' % opt)
+                self._error('no such option: %s' % (arg if arg.startswith('--') else opt))
             if option.takes_value:
                 # Any characters left in arg?  Pretend they're the
                 # next arg, and stop consuming characters of arg.
