@@ -3,7 +3,9 @@ import sys
 from collections import deque
 
 from ._compat import text_type, open_stream, get_streerror, string_types, \
-     PY2, get_best_encoding, binary_streams, text_streams, filename_to_ui
+     PY2, binary_streams, text_streams, filename_to_ui, \
+     auto_wrap_for_ansi, strip_ansi, isatty, _default_text_stdout, \
+     is_bytes
 
 if not PY2:
     from ._compat import _find_binary_writer
@@ -12,8 +14,12 @@ if not PY2:
 echo_native_types = string_types + (bytes, bytearray)
 
 
+def _posixify(name):
+    return '-'.join(name.split()).lower()
+
+
 def unpack_args(args, nargs_spec):
-    """Given an iterable of arguments and an iterable of nargs specifications
+    """Given an iterable of arguments and an iterable of nargs specifications,
     it returns a tuple with all the unpacked arguments at the first index
     and all remaining arguments as the second.
 
@@ -50,7 +56,7 @@ def unpack_args(args, nargs_spec):
             rv.append(_fetch(args))
         elif nargs > 1:
             x = [_fetch(args) for _ in range(nargs)]
-            # If we're reversed we're pulling in the arguments in reverse
+            # If we're reversed, we're pulling in the arguments in reverse,
             # so we need to turn them around.
             if spos is not None:
                 x.reverse()
@@ -61,7 +67,7 @@ def unpack_args(args, nargs_spec):
             spos = len(rv)
             rv.append(None)
 
-    # spos is the position of the wildcard (star).  If it's not None
+    # spos is the position of the wildcard (star).  If it's not `None`,
     # we fill it with the remainder.
     if spos is not None:
         rv[spos] = tuple(args)
@@ -122,11 +128,13 @@ class LazyFile(object):
     files for writing.
     """
 
-    def __init__(self, filename, mode='r', encoding=None, errors='strict'):
+    def __init__(self, filename, mode='r', encoding=None, errors='strict',
+                 atomic=False):
         self.name = filename
         self.mode = mode
         self.encoding = encoding
         self.errors = errors
+        self.atomic = atomic
 
         if filename == '-':
             self._f, self.should_close = open_stream(filename, mode,
@@ -136,7 +144,7 @@ class LazyFile(object):
                 # Open and close the file in case we're opening it for
                 # reading so that we can catch at least some errors in
                 # some cases early.
-                open(filename, mode, encoding, errors).close()
+                open(filename, mode).close()
             self._f = None
             self.should_close = True
 
@@ -158,7 +166,8 @@ class LazyFile(object):
         try:
             rv, self.should_close = open_stream(self.name, self.mode,
                                                 self.encoding,
-                                                self.errors)
+                                                self.errors,
+                                                atomic=self.atomic)
         except (IOError, OSError) as e:
             from .exceptions import FileError
             raise FileError(self.name, hint=get_streerror(e))
@@ -186,39 +195,66 @@ class LazyFile(object):
 
 def echo(message=None, file=None, nl=True):
     """Prints a message plus a newline to the given file or stdout.  On
-    first sight this looks like the print function but it has improved
-    support for handling unicode and binary data that does not fail no
+    first sight, this looks like the print function, but it has improved
+    support for handling Unicode and binary data that does not fail no
     matter how badly configured the system is.
 
-    Primarily it means that you can print binary data as well as unicode
+    Primarily it means that you can print binary data as well as Unicode
     data on both 2.x and 3.x to the given file in the most appropriate way
     possible.  This is a very carefree function as in that it will try its
     best to not fail.
 
+    In addition to that, if `colorama`_ is installed, the echo function will
+    also support clever handling of ANSI codes.  Essentially it will then
+    do the following:
+
+    -   add transparent handling of ANSI color codes on Windows.
+    -   hide ANSI codes automatically if the destination file is not a
+        terminal.
+
+    .. _colorama: http://pypi.python.org/pypi/colorama
+
+    .. versionchanged:: 2.0
+       Starting with version 2.0 of click, the echo function will work
+       with colorama if it's installed.
+
     :param message: the message to print
     :param file: the file to write to (defaults to ``stdout``)
     :param nl: if set to `True` (the default) a newline is printed afterwards.
     """
     if file is None:
-        file = sys.stdout
+        file = _default_text_stdout()
 
+    # Convert non bytes/text into the native string type.
     if message is not None and not isinstance(message, echo_native_types):
         message = text_type(message)
 
+    # If there is a message, and we're in Python 3, and the value looks
+    # like bytes, we manually need to find the binary stream and write the
+    # message in there.  This is done separately so that most stream
+    # types will work as you would expect.  Eg: you can write to StringIO
+    # for other cases.
+    if message and not PY2 and is_bytes(message):
+        binary_file = _find_binary_writer(file)
+        if binary_file is not None:
+            file.flush()
+            binary_file.write(message)
+            if nl:
+                binary_file.write(b'\n')
+            binary_file.flush()
+            return
+
+    # ANSI-style support.  If there is no message or we are dealing with
+    # bytes nothing is happening.  If we are connected to a file we want
+    # to strip colors.  If we have support for wrapping streams (windows
+    # through colorama) we want to do that.
+    if message and not is_bytes(message):
+        if not isatty(file):
+            message = strip_ansi(message)
+        elif auto_wrap_for_ansi is not None:
+            file = auto_wrap_for_ansi(file)
+
     if message:
-        if PY2:
-            if isinstance(message, text_type):
-                encoding = get_best_encoding(file)
-                message = message.encode(encoding, 'replace')
-        elif isinstance(message, (bytes, bytearray)):
-            binary_file = _find_binary_writer(file)
-            if binary_file is not None:
-                file.flush()
-                binary_file.write(message)
-                if nl:
-                    binary_file.write(b'\n')
-                binary_file.flush()
-                return
         file.write(message)
     if nl:
         file.write('\n')
@@ -262,7 +298,7 @@ def format_filename(filename, shorten=False):
     """Formats a filename for user display.  The main purpose of this
     function is to ensure that the filename can be displayed at all.  This
     will decode the filename to unicode if necessary in a way that it will
-    not fail.  Optionally it can shorten the filename to not include the
+    not fail.  Optionally, it can shorten the filename to not include the
     full path to the filename.
 
     :param filename: formats a filename for UI display.  This will also convert
@@ -273,3 +309,54 @@ def format_filename(filename, shorten=False):
     if shorten:
         filename = os.path.basename(filename)
     return filename_to_ui(filename)
+
+
+def get_app_dir(app_name, roaming=True, force_posix=False):
+    r"""Returns the config folder for the application.  The default behavior
+    is to return whatever is most appropriate for the operating system.
+
+    To give you an idea, for an app called ``"Foo Bar"``, something like
+    the following folders could be returned:
+
+    Mac OS X:
+      ``~/Library/Application Support/Foo Bar``
+    Mac OS X (POSIX):
+      ``~/.foo-bar``
+    Unix:
+      ``~/.config/foo-bar``
+    Unix (POSIX):
+      ``~/.foo-bar``
+    Win XP (roaming):
+      ``C:\Documents and Settings\<user>\Local Settings\Application Data\Foo Bar``
+    Win XP (not roaming):
+      ``C:\Documents and Settings\<user>\Application Data\Foo Bar``
+    Win 7 (roaming):
+      ``C:\Users\<user>\AppData\Roaming\Foo Bar``
+    Win 7 (not roaming):
+      ``C:\Users\<user>\AppData\Local\Foo Bar``
+
+    .. versionadded:: 2.0
+
+    :param app_name: the application name.  This should be properly capitalized
+                     and can contain whitespace.
+    :param roaming: controls if the folder should be roaming or not on Windows.
+                    Has no affect otherwise.
+    :param force_posix: if this is set to `True` then on any POSIX system the
+                        folder will be stored in the home folder with a leading
+                        dot instead of the XDG config home or darwin's
+                        application support folder.
+    """
+    if sys.platform.startswith('win'):
+        key = roaming and 'APPDATA' or 'LOCALAPPDATA'
+        folder = os.environ.get(key)
+        if folder is None:
+            folder = os.path.expanduser('~')
+        return os.path.join(folder, app_name)
+    if force_posix:
+        return os.path.join(os.path.expanduser('~/.' + _posixify(app_name)))
+    if sys.platform == 'darwin':
+        return os.path.join(os.path.expanduser(
+            '~/Library/Application Support'), app_name)
+    return os.path.join(
+        os.environ.get('XDG_CONFIG_HOME', os.path.expanduser('~/.config')),
+        _posixify(app_name))
